;*******************************************************************************
;*
;*      Titel:  Hauptrogramm zur Nutzung der Vendor USB Bibliothek
;*              (weigu_firmware_vendor_v1_3_a.asm)
;*      Datum:  07/09/10                Version: 1.3 (07/04/16)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      
;*      Prozessor: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
;*                 (siehe Zuweisungen)
;*                 
;*      Quarz:     8 oder 16 MHz  (siehe Zuweisungen in der USB-Bibliothek)
;*      
;*      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
;*     
;*      Eingaenge:   ADC0 (PF0) analog
;*      Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS
;*      
;*      AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
;*
;*      Eingaenge:   Temperatursensor an ADC0 (PF0) analog
;*      Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
;*      
;*      Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
;*
;*      Eingaenge:   PC4 
;*      Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS
;*                  
;*      Informationen zur Funktionsweise: 
;*      
;*      Hauptprogramm zur Nutzung der Vendor-Bibliothek
;*      Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
;*      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
;*      abgewickelt. Alle USB-Routinen und Unterprogramme (wie zB. "USBINI")
;*      befinden sich in der Bibliothek "SR_USB_VENDOR_v1_2.asm"
;*      
;*      Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung. 
;*      
;*      Beim Endpunkt 1 (IN) fragt der PC Daten an (hier ADC-Wert 2 Byte).
;*      Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
;*      neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
;*      Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP2_BUF) abgelegt.
;*      
;*      Ueber den Endpunkt 2 (OUT) sendet der PC Daten (hier ein Byte mit
;*      Informationen füe die LEDS (5 Bit)). Die Hauptschleife erkennt an einem 
;*      Flag (EP2_FLAG) wenn Daten vorliegen. Die Daten liegen im Buffer
;*      (EP2_BUF). Die Variable EP2_CNT gibt an wie viele Bytes gesendet wurden.
;*      
;*      Weitere Dokumentation auf www.weigu.lu/b/usb/
;*      
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;       Einbinden der controllerspez.Definitionsdatei und Auswahl des Boards
;-------------------------------------------------------------------------------
.NOLIST                         ;List-Output ausschalten

;Hier bitte auswaehlen welches Board benutzt wir!!
;Teensy 2.0: ATmega32u4  16 MHz
;AT90USBKEY: AT90USB1287  8 MHz
;Teensy 1.0: AT90USB162  16 MHz

.EQU TEENSY2    = 1             ;eines der Flags auf 1, die anderen auf 0
.EQU AT90USBKEY = 0
.EQU TEENSY1    = 0

.IF TEENSY2 == 1
.INCLUDE "m32u4def.inc"         ;AVR-Definitionsdatei einbinden
.EQU CRYSTAL = 16
.ENDIF

.IF AT90USBKEY == 1
.INCLUDE "usb1287def.inc"
.EQU CRYSTAL = 8
.ENDIF

.IF TEENSY1 == 1
.INCLUDE "usb162def.inc"
.EQU CRYSTAL = 16
.ENDIF

.LIST                           ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
.ORG    SRAM_START
;Globale Variable
BLINKFLAG:      .BYTE 1                 ;Blink On Off (0 = Off 1 = On)

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG    0x0000                  ;Programm beginnt an der FLASH-Adresse 0x0000
RESET1: rjmp    INIT            ;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;       Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        rjmp    ISRGEN          ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        rjmp    ISRCOM          ;Springe zur ISR des EP COMmunication Interrupt
.ORG    OC1Aaddr                ;Timer/Counter1 Compare Match A
        rjmp    ISRTC1          ;springe zur ISR Timer1

;-------------------------------------------------------------------------------
;       Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero  = r15             ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF    Tmp1  = r16             ;Register 16 dient als erster Zwischenspeicher
.DEF    Tmp2  = r17             ;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1  = r18             ;Register 18 dient als Zaehler
.DEF    WL    = r24             ;Register 24 und 25 dienen als universelles
.DEF    WH    = r25             ;Doppelregister W und zur Parameteruebergabe
.DEF    W     = r24

        ;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
        ldi     Tmp1,HIGH(RAMEND)       ;RAMEND (SRAM) ist in der Definitions-
        out     SPH,Tmp1                ;datei festgelegt
        ldi     Tmp1,LOW(RAMEND)
        out     SPL,Tmp1

        ;Im Originalzustand ist das Fusebit CKDIV8 gesetzt, was eine Teilung
        ;durch 8 der Frequenz bewirkt
        ldi     Tmp1,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,Tmp1       ;ueberschreiben
        clr     Tmp1             ;CLKPR = 0, Teiler = 1
        sts     CLKPR,Tmp1

        ;Timer 1 initialisieren
        ;Interrupt zwei mal pro Sekunde
        ;Vorteiler 1024, Zaehlschritte 7812-1 ergeben bei 16 MHz 1 Sekunde
        clr     Tmp1
        sts     TCCR1A,Tmp1
        ldi     Tmp1,0b00001101 ;WGM12 = 1 Teiler = 1024
        sts     TCCR1B,Tmp1
.IF CRYSTAL == 8
        ldi     Tmp1,HIGH(3905) ;8MHz
        ldi     Tmp2,LOW(3905)  ;16MHz
.ELSE        
        ldi     Tmp1,HIGH(7811)
        ldi     Tmp2,LOW(7811)
.ENDIF        
        sts     OCR1AH,Tmp1     ;Achtung High zuerst!
        sts     OCR1AL,Tmp2
        ldi     Tmp1,0x02       ;OCI1A Interrupt erlauben
        sts     TIMSK1,Tmp1

.IF TEENSY1 == 1                ;Der AT90USB162 besitzt keinen A/D-Wandler
                                ;Es wird deshalb PC4 als Eingang eingelesen                       
        cbi     DDRC,4          ;PC4 als Eingang
        sbi     PORTC,4         ;Pull-Up
.ELSE
        ;ADC initialisieren
        ldi     Tmp1,0b01000000 ;REFS = 01 REF = AVCC
        sts     ADMUX,Tmp1      ;ADLAR = 0 rechtsbuendig
                                ;MUX = 00000 unsymetrisch ADC0
        ldi     Tmp1,0b10000111 ;ADEN = 1 (ADC einschalten)
        sts     ADCSRA,Tmp1     ;ADPS = 111 andere Bits per default = 0
                                ;16MHz/128 = 125kHz, 8MHz/128 = 62,5kHz
.ENDIF

.IF AT90USBKEY == 1
        sbi     DDRD,4          ;PD4 rot   4 LEDs on Board 
        sbi     DDRD,5          ;PD5 gruen
        sbi     DDRD,6          ;PD6 gruen
        sbi     DDRD,7          ;PD7 rot
.ELSE
        sbi     DDRD,6          ;PD6 Blink LED   Teensy2 on Board
        sbi     DDRB,0          ;PB0-3 LEDs
        sbi     DDRB,1
        sbi     DDRB,2
        sbi     DDRB,3
.ENDIF

        ;globale Variable initialisieren
        ldi     Tmp1,0x01        ;Blinken ein
        sts     BLINKFLAG,Tmp1

        rcall   USBINI          ;befindet sich in der USB-Bibliothek

        ldi     Tmp1,2          ;ADC liefert 2 Byte!
        sts     EP1_CNT,Tmp1

;-------------------------------------------------------------------------------
;        Hauptprogramm
;-------------------------------------------------------------------------------
.IF TEENSY1 == 1                ;Der AT90USB162 besitzt keinen A/D-Wandler
MAIN:   cli
        sbic    PINC,4          ;Es wird deshalb PC4 als Eingang eingelesen
        rjmp    PC4SET
        
        sts     EP1_BUF+0,Zero
        sts     EP1_BUF+1,Zero
        rjmp    EP1SETF
PC4SET: ldi     Tmp1,LOW(1023)
        sts     EP1_BUF+0,Tmp1
        ldi     Tmp1,HIGH(1023)
        sts     EP1_BUF+1,Tmp1
EP1SETF: sei

.ELSE
MAIN:   lds     Tmp1,ADCSRA     ;sbi ADCSRA,ADSC ;eine AD-Wandlung ausloesen
        ori     Tmp1,0x40
        sts     ADCSRA,Tmp1

ADCLOOP: lds    Tmp1,ADCSRA     ;Polling von ADSC (0 =  Wandlung fertig)
        andi    Tmp1,0x40
        brne    ADCLOOP

        cli
        lds     Tmp1,ADCL       ;ADC-Wert im EP1-SRAM Buffer abspeichern
        sts     EP1_BUF+0,Tmp1
        lds     Tmp1,ADCH
        sts     EP1_BUF+1,Tmp1
        sei
.ENDIF
        ldi     Tmp1,1          ;EP1 Flag setzen weil Daten vorhanden
        sts     EP1_FLAG,Tmp1

        lds     Tmp1,EP2_Flag   ;neue Daten im EP2 vorhanden?
        cpi     Tmp1,1
        brne    BLINKT

        lds     Tmp1,EP2_BUF+0  ;Byte einlesen und PD6 ausmaskieren (Blink Flag)
        mov     Tmp2,Tmp1
        andi    Tmp1,0x40
        sts     BLINKFLAG,Tmp1
        andi    Tmp2,0x0F       ;4 LEDs ein- bzw. aus
.IF AT90USBKEY == 1             ;4LEDs (oberstes Nibble PD4-7)
        swap    Tmp2
        out     PORTD,Tmp2
.ELSE
        out     PORTB,Tmp2
.ENDIF
        sts     EP2_FLAG,Zero   ;EP2 Flag loeschen

BLINKT: lds     Tmp1,BLINKFLAG
        cpi     Tmp1,0
        breq    NOBLINK

        ldi     Tmp1,0x02       ;OCI1A Interrupt erlauben
        sts     TIMSK1,Tmp1
        rjmp    MAIN            ;Endlosschleife
NOBLINK: sts    TIMSK1,Zero
        rjmp    MAIN            ;Endlosschleife

;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;Timer 1 ISR Interrupt jede Sekunde
ISRTC1: push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16

        sbi     PIND,6          ;Toggle LED

        pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti            

.INCLUDE "lib/SR_USB_VENDOR_V1_3.asm" ;USB Bibliothek

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT                           ;Ende des Quelltextes
