/*! \file usb_srs_vendor_v1_2.c
********************************************************************************
 small usb library for avrs (mega32u4, at90usb162, at90usb1287)
 using vendor class with bulk transfers
 2 endpoints (1*IN 1*OUT) with 64 Byte
 
 www.weigu.lu/b/usb

 (inspired by the lib from S. Salewski (http://http://www.ssalewski.de)

 \author weigu www.weigu.lu
 
******************************************************************************/

/* Copyright (c) 2013   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */

#include <stdint.h>
#include <avr/pgmspace.h> 
#include <avr/interrupt.h> 
#include "usb_srs_vendor_v1_2.h"


/*! \brief USB General Interrupt (S253)*/ 
ISR(USB_GEN_vect)
{
  if (UDINT & (1<<EORSTI)) // End Of ReSeT?
  {
    CBI (UDINT,EORSTI); // sperre EORSTI
    // Initialisiere EP0
    usb_init_endpoint(0,Ep0_ty, Ep0_di, Ep0_si, Ep0_ba);    
    SBI(UEIENX,RXSTPE);  
   }
}

/*! \brief Endpoint/Pipe Communication Interrupt Service Routine (S254) */    
ISR(USB_COM_vect)
{
   // EP Nummer durch Bitposition in UEINT gekennzeichnet 2^EP = 1
  switch (UEINT)
  {
    //EP0: wenn Setup-Paket von PC angekommen, dieses behandeln (Enumeration)
    case 1: UENUM = 0; if (UEINTX & (1<<RXSTPI)) usb_ep0_setup(); break;
    //EP1: IN
    case 2: UENUM = 1; usb_ep1_in(); break;
    //Ep2: OUT
    case 4: UENUM = 2; usb_ep2_out(); break;
    
    //// hier erweitern 
    //case 8: //UENUM = 3;  ///      break;
    
    default: break;
  }
}

/*! \brief USB-Aktivierung (Full-Speed 12Mbit/s) und Interrupts erlauben */
void usb_init_device(void)
{
#if defined(__AVR_ATmega32U4__)
  UHWCON = (1<<UVREGE); //Regelkreis zur Versorgung der D+ und D- Leitung einschalten
#endif        
#if defined(__AVR_AT90USB1287__) // Der ATMEL Bootloader nutzt einige Interrupts
  USBCON = (1<<FRZCLK);   
  OTGIEN = 0;                   // Deaktiviere diese Interrupts damit sauberer Start
  UDIEN  = 0;
  UHIEN  = 0;
  UEIENX = 0;
  UPIENX = 0;
  UHWCON = (1<<UIMOD);          //Enable Device Modus und Regelkreis zur Versorgung
  UHWCON = ((1<<UIMOD) | (1<<UVREGE)); // der D+ und D- Leitung einschalten
#endif
#if defined(__AVR_AT90USB162__)
  USBCON = ((1<<USBE) | (1<<FRZCLK)); // 0xA0 ;USB Sender/Empf. und Takt einschalten
#else
  USBCON = ((1<<USBE) | (1<<FRZCLK) | (1<<OTGPADE)); //0xB0 USB Sender/Empf.,
                                                    // Takt und VBUS-Leitung einschalten
#endif  
  USBCON &= ~(1<<FRZCLK); // FRZCLK=0, noetig, da sonst kein WAKEUP Interrupt
  USBCON |= (1<<FRZCLK);  // FRZCLK=1 Strom sparen
  // Starte PLL
  PLLCSR = PLLPRE;      // PLL Vorteiler fuer den Quarz (siehe Zuweisungen)
  PLLCSR |= (1<<PLLE);  // starte PLL (PLLEnable=1)
  while (!(PLLCSR &(1<<PLOCK)));     // Warte bis PLOCK = 1 (PLL eingerastet)
  USBCON &= ~(1<<FRZCLK); // FRZCLK=0, aktiviere den Takt
  UDCON = 0; // Attach: Verbinde das Device
  UDIEN = (1<<EORSTE);  //0x08 erlaube End Of ReSeT Interrupt

  // Endpunkt Buffer, Flags und Zaehler loeschen
  for (int i = 0; i < 64; i++)
  {
    Ep2_buf[i] = 0x00;
    Ep1_buf[i] = 0x00;
  }
  Ep1_flag = 0;
  Ep2_flag = 0;
  Ep1_cnt = 0;
  Ep2_cnt = 0;

#if defined FTDI  
  Ep1_buf[0] = 0x01;  //modem status byte (B0 reserved must be 1)
  Ep1_buf[1] = 0x60;
  resetflag = 0;        // Flag Kommando Reset Device
  setmodemflag = 0;     // Flag Kommando Set Modem Control Register
  setbaudflag = 0;      // Flag Kommando Set Baudrate
  setdataflag = 0;      // Flag Kommando Set Data Format
#endif  
  sei();     // Interrupts global erlauben
}

/*! \brief Aktiviere Endpunkt
    \PARAM usb_init_endpoint()
    nu EP_Number: 0...6
    ty Type:      0 = Control, 1 = Isochron, 2 = Bulk 3 = Interrupt
    di Direction: 0 = OUT, 1 = IN (ausser Control)
    si Size:      0 = 8 Bytes, 1 = 16 Bytes, 2 = 32 Bytes, 3 = 64 Bytes, 
                 4 = 128 Bytes (nur EP1), 5 = 256 Bytes (EP1), andere reserviert
    ba Bank:      0 = 1 Bank, 1 = 2 Baenke, andere res.
    Flussdiagramm zum Aktivieren eines EP S269 hier keine Fehlerkontrolle
    */
void usb_init_endpoint(uint8_t nu,uint8_t ty,uint8_t di,uint8_t si,uint8_t ba)
{
  UENUM = nu; // Waehle Endpoint 
  SBI(UECONX,EPEN); // Aktiviere EP
  UECFG0X = ((ty << 6) | (di)); // TYPE + DIR festlegen
  UECFG1X = ((si << 4) | (ba << 2)); // SIZE + BanK festlegen
  SBI(UECFG1X,ALLOC); // Speicher zuweisen    
}

/*! \brief 
    Wird aufgerufen, wenn ein Received SETUP Interrupt (RXSTPI in UEINTX)
    auftritt (EP0 S272).
    */ 
void usb_ep0_setup(void)
{
  uint8_t bmRequestType;
  uint8_t bRequest;
  uint8_t wValue_l;
  uint8_t wValue_h;
  uint8_t wIndex_l;
  uint8_t wIndex_h;
  uint8_t wLength_l;
  uint8_t wLength_h;
  uint8_t des_bytes;

  // Ab hier folgen die Deskriptortabellen im Flash 
  /*** Device Descriptor ***/
  static const uint8_t PROGMEM dev_des[] = 
  {
    18,       // bLength = 18 (0x12), Groesse des Deskriptors in Byte
    0x01,     // bDescriptorType = 0x01, Konstante Device = 1
    0x10,0x01,// bcdUSB = 0x0200, USB_Spec2_0
    0x00,     // bDeviceClass = 0x00, Klassencode: definiert auf Interface Level
    0x00,     // bDeviceSubClass = 0x00
    0x00,     // bDeviceProtocoll = 0x00
    Ep0_fs,   // bMaxPacketSize0 = EP0FS, max. Paketgroesse EP0 (hier 8 B)
#if defined FTDI
//  0x7D,0x0C,// idVendor = 0x0C7D Elektor
    0x03,0x04,// idVendor = 0x0403, FTDI;
//  0x05,0x00,// idProduct = 0x0005  Elektor
    0x01,0x60,// idProduct = 0x6001 FTDI
    0x00,0x06,// bcdDevice = 0x0100, Release Nummer Geraet
#else
    0xeb,0x03,// idVendor = 0x03eb, Atmel Code durch usb.org vergeben
    0x01,0x00,// idProduct = 0x0001, Produkt ID
    0x01,0x00,// bcdDevice = 0x0100, Release Nummer Geraet 
#endif
    Manu_i,   // iManufacturer = Index fuer String-Deskriptor Hersteller
    Prod_i,   // iProduct = Index fuer String-Deskriptor    Produkt
    Seri_i,   // iSerialNumber = Index fuer String-Deskriptor Seriennummer
    0x01      // bNumConfigurations = 1, Anzahl moeglicher Konfigurationen*/
  };     
  /*** Configurations Descriptor ***/
  static const uint8_t PROGMEM conf_des[] =  
  {
    9,        //bLength = 0x09, Groesse Deskriptor in Byte
    0x02,     //bDescriptorType = 0x02, Konstante Configuration = 2
    low(wTotalLength),high(wTotalLength),  //wTotalLength, Laenge Configuration
              //Deskriptor + alle untergeordnete Deskriptoren.
    0x01,     //bNumInterfaces = 1
    0x01,     //bConfigurationValue = 1, darf nicht Null sein (sonst geht
              //Geraet in den nicht-konfigurierten Zustand))
    0,        //iConfiguration = 0, Index fuer Str.-Deskr. Conf. (0=kein Txt)
    0x80,     //bmAttributes = 0x80,bus-powered, kein remote wakeup Bit 7=1
    250,      //MaxPower = 250(dezimal), in 2mA Schritten! (500mA)
    /*** Interface Descriptor ***/
    9,        //bLength = 0x09, Groesse Deskriptor in Byte
    0x04,     //bDescriptorType = 0x04, Konstante Interface = 4
    0,        //bInterfaceNumber = 0;
    0,        //bAlternateSetting = 0;
    Nr_eps,   //bNumEndpoints = USB_Endpoints;
    0xFF,     //bInterfaceClass = 0xFF, Klassencode: anbieterspez. (0xFF)
    0xFF,     //bInterfaceSubClass = 0xFF, Unterklassencode anbietersp. (0xFF)
    0xFF,     //bInterfaceProtocol = 0xFF, Protokollcode anbieterspez. (0xFF)
    Intf_i,   //iInterface = 0, Index fuer String-Deskr. Interf. (0=kein Txt)
    /*** EP1 Descriptor ***/
    7,        //bLength = 0x07, Groesse Deskriptor in Byte
    0x05,     //bDescriptorType = 0x05, Konstante Endpoint = 5
    0x81,     //bEndpointAddress = 0x81, Bit 7 = 1 (IN) Bit 0-3 EP-Nummer
    0x02,     //bmAttributes = 0x02, Transfertyp = Bulk (2) (c=0, iso=1, int=3)
    low(Ep1_fs),high(Ep1_fs),    //wMaxPacketSize = EP1_FIFO_Size;
    0,        //bInterval = 0, Bulk EP, kein polling             
    /*** EP2 Descriptor ***/
    7,        //bLength = 0x07, Groesse Deskriptor in Byte
    0x05,     //bDescriptorType = 0x05, Konstante Endpoint = 5
    0x02,     //bEndpointAddress = 0x02, Bit 7 = 0 (OUT) Bit 0-3 EP-Nummer
    0x02,     //bmAttributes = 0x02, Transfertyp = Bulk (2) (c=0, iso=1, int=3)
    low(Ep2_fs),high(Ep2_fs),    //wMaxPacketSize = EP2_FIFO_Size;
    0         //bInterval = 0, Bulk EP, kein polling 
  };                    
  /*** Language Descriptor ***/      
  static const uint8_t PROGMEM lang_des[] = 
  {
    4,        //bLength = 0x04, Groesse Deskriptor in Byte
    0x03,     //bDescriptorType = 0x03, Konstante String = 3
    0x09,0x04 //wLANGID[0] = 0x0409 = English USA (Supported Lang. Code 0) 
  };
  /*** Manufacturer Descriptor ***/          
  static const uint8_t PROGMEM manu_des[] = 
  {
    18,       //bLength = 0x04, Groesse Deskriptor in Byte
    0x03,     //bDescriptorType = 0x03, Konstante String = 3
              //bString = Unicode Encoded String (16 Bit!)
    'W',0,'E',0,'I',0,'G',0,'U',0,'.',0,'L',0,'U',0 
  };        
  /*** Product Descriptor ***/       
  static const uint8_t PROGMEM prod_des[] = 
  {
 /*   16,       //bLength = 0x04, Groesse Deskriptor in Byte
    0x03,     //bDescriptorType = 0x03, Konstante String = 3
              //bString = Unicode Encoded String (16 Bit!)
    'B',0,'S',0,'3',0,'2',0,'5',0,'U',0,'W',0*/

    22,       //bLength = 0x22, Groesse Deskriptor in Byte
    0x03,     //bDescriptorType = 0x03, Konstante String = 3
              //bString = Unicode Encoded String (16 Bit!)
#if defined FTDI    
    'U',0,'S',0,'B',0,'-',0,'S',0,'E',0,'R',0,'I',0,'A',0,'L',0
#else
    'U',0,'S',0,'B',0,'-',0,'V',0,'E',0,'N',0,'D',0,'O',0,'R',0
#endif
  };        
  /*** Serial Descriptor ***/      
  static const uint8_t PROGMEM seri_des[] = 
  {
    12,       //bLength = 0x12, Groesse Deskriptor in Byte
    0x03,     //bDescriptorType = 0x03, Konstante String = 3
              //bString = Unicode Encoded String (16 Bit!) 
    'W',0,'V',0,'1',0,'2',0,'c',0,
  };
  
  
  /* hier erweitern falls weitere EPs benoetigt werden*/
  
  bmRequestType = UEDATX; 
  bRequest = UEDATX; 
  wValue_l = UEDATX; 
  wValue_h = UEDATX; 
  wIndex_l = UEDATX; 
  wIndex_h = UEDATX;
  wLength_l = UEDATX; 
  wLength_h = UEDATX; 
  
  CBI(UEINTX, RXSTPI); //'ACK SU-Paket (ACK für SETUP Packet und FIFO löschen)
                    
  if ((bmRequestType & 0x60) == 0) // Type = Standard Device Request
  {
    switch (bRequest)
    {
      case 0x00: //GET_STATUS 3 Phasen (optional kann geloescht werden!)
        UEDATX = 0;
        UEDATX = 0;
        CBI(UEINTX,TXINI); //Daten absenden (ACK) und FIFO löschen
        while (!(UEINTX & (1 << RXOUTI)));  //Warten bis ZLP vom PC
        CBI(UEINTX, RXOUTI);//Flag loeschen
        break;
      case 0x05: //SET_ADDRESS 2 Phasen (keine Datenphase)
        UDADDR = (wValue_l & 0x7F); // Speichere Adresse UADD (ADDEN = 0)
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272 
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        SBI(UDADDR, ADDEN); // Adresse aktivieren
        break;
      case 0x06: //GET_DESCRIPTOR 3 Phasen Transfer 
        {
          switch (wValue_h)
          {
            case 1: // Device-Descriptor          
              des_bytes = pgm_read_byte(&dev_des[0]);	      
              usb_send_descriptor((uint8_t*) dev_des,des_bytes);         
              break;
            case 2: // Configuration-Descriptor
              des_bytes = wLength_l;
	      //Manchmal erfragt Windows unsinnig hohe Werte (groesser als 256
	      //Byte). Dadurch steht dann im Lowbyte ev. ein falscher Wert
	      //Es wird hier davon ausgegangen, dass nicht mehr als 256 Byte 
	      //angefragt werden können.
              if (wLength_h || (wLength_l > wTotalLength) || (wLength_l == 0)) 
                 des_bytes = wTotalLength;
              usb_send_descriptor((uint8_t*) conf_des,des_bytes);
              break;
            case 3: // String-Descriptor
              switch (wValue_l)
              {
                case Lang_i:
                  des_bytes = pgm_read_byte(&lang_des[0]);
                  usb_send_descriptor((uint8_t*) lang_des,des_bytes);
                  break;
                case Manu_i:
                  des_bytes = pgm_read_byte(&manu_des[0]);
                  usb_send_descriptor((uint8_t*) manu_des,des_bytes);
                  break;
                case Prod_i:
                  des_bytes = pgm_read_byte(&prod_des[0]);
                  usb_send_descriptor((uint8_t*) prod_des,des_bytes);
                  break;
                case Seri_i:
                  des_bytes = pgm_read_byte(&seri_des[0]);
                  usb_send_descriptor((uint8_t*) seri_des,des_bytes);
                  break;              
                default: break;
              }
              break;
            default: break; 
          }      
          /*if (!(UEINTX & (1 << RXOUTI))) // es sind noch restliche Bytes abzuschicken
          {
            CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12
            while (!(UEINTX & (1 << RXOUTI))); //Warten bis ACK(ZLP) vom PC angekommen
          }   
          CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen */
        }
        break;
      case 0x09: //SET_CONFIGURATION 2 Phasen (keine Datenphase)
        for (uint8_t i=Nr_eps;i==1;i--) //Benutzte Endpoints abschalten
        {                                //Rückwärts (siehe Speicherbelegung)
          UENUM = i; // Waehle EP i  
          CBI(UECONX, EPEN); // Schalte Endpunkt ab
          CBI(UECFG1X, ALLOC); // Gib den Speicher frei
        }
        //Initialisierung Endpunkt1  (IN-EP)
        usb_init_endpoint(1,Ep1_ty, Ep1_di, Ep1_si, Ep1_ba);
	SBI(UEIENX, NAKINE);  //NAK INterrupts für IN Endpunkte erlauben
        //Initialisierung Endpunkt 2 (OUT-EP)
        usb_init_endpoint(2,Ep2_ty, Ep2_di, Ep2_si, Ep2_ba);
        SBI(UEIENX, RXOUTE);  //RX OUT Interrupts für OUT Endpunkte erlauben
        
        ////hier erweitern
 
        UENUM = 0; // Waehle EP0  
        CBI(UEINTX, TXINI); //sende ZLP (Erfolg, löscht EP-Bank)
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        break;
      default: SBI(UECONX,STALLRQ); break;
    }
  }
# if defined FTDI  
  // FTDI Kommandos
  if ((bmRequestType & 0x60) == 0x40) // Vendor Herstellerspezifische Anfrage
  {
    switch (bRequest)
    {
      case 0: // RESET the Port
	resetflag = 1;
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;
      case 1: // Set the modem control register
     	setmodemflag = 1; //Kommando Set Modem Control Register
     	setmodemdata_l =wValue_l;
	setmodemdata_h =wValue_h;
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;
      case 2: // Set flow control register
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;	
      case 3: // Set Baudrate
        setbaudflag = 1;  //Kommando Set Baudrate
        setbauddata_l = wValue_l;
	setbauddata_h = wValue_h;
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen	
        break;
      case 4: // Set Data Format
       	setdataflag = 1;  //Kommando Set Data Format
        setdatadata_l = wValue_l;
	setdatadata_h = wValue_h;
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;
      case 5: // Retrieve current value of modem status register
        UEDATX = 0x10;          //Setze Standard Wert
        CBI(UEINTX,TXINI); //Daten absenden (ACK) und FIFO löschen
        while (!(UEINTX & (1 << RXOUTI)));  //Warten bis ZLP vom PC
        CBI(UEINTX, RXOUTI);//Flag loeschen
        break;
      case 6: // Set the event character
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;
      case 7: // Set the error character
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen
        break;
      case 9: // Set latency timer
        CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12 S272
        while (!(UEINTX & (1<<TXINI)));// Warte bis fertig (EP Bank wieder frei)
        CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen 
        break;
      case 10: // Get latency timer
        UEDATX = 16;
        CBI(UEINTX,TXINI); //Daten absenden (ACK) und FIFO löschen
        while (!(UEINTX & (1 << RXOUTI)));  //Warten bis ZLP vom PC
        CBI(UEINTX, RXOUTI);//Flag loeschen
        break;
      default: SBI(UECONX,STALLRQ); break;
    }
  }
#endif   
  else SBI(UECONX,STALLRQ); // Kein Standard Request, erfrage STALL Antwort
}

/*! \brief Sende Deskriptor zum PC (22.14 IN-EP management S 275)
    Es werden nur so viele Bytes gesendet wie angefragt. Falls PC in dieser 
    Phase (bei Control Transaktion) abbrechen moechte, so sendet er ein ZLP 
    Paket (2.14.1.1 S 276). 
    Fuelle FIFO (gegebenfalls mehrmals) und sende Daten.
     */
void usb_send_descriptor(uint8_t de[] ,uint8_t db)
{
  for (uint16_t i=1;i<=db;i++) //Achtung Überlauf bei uint8_t!!
  {
    if (UEINTX & (1 << RXOUTI)) return;// PC will abbrechen!
    UEDATX = pgm_read_byte(&de[i-1]); // Schreibe Byte ins FIFO
    //immer nach 8 Bytes das Paket absenden und Speicherbank löschen
    if ((i % Ep0_fs) == 0) // falls FIFO gefuellt (8 Byte)
    {
      CBI(UEINTX,TXINI); //IN Paket (FIFO) senden, EP Bank loeschen 22.12 S272 
      while (!(UEINTX & ((1<<RXOUTI) | (1<<TXINI)))); // Warte bis Bank frei oder ACK Host
    }
  }
  if (!(UEINTX & (1 << RXOUTI))) // es sind noch restliche Bytes abzuschicken
  {
      CBI(UEINTX,TXINI); //IN Paket (ZLP) senden, EP Bank loeschen 22.12
      while (!(UEINTX & (1 << RXOUTI))); //Warten bis ACK(ZLP) vom PC angekommen
  }   
  CBI(UEINTX, RXOUTI);// Handshake um Interrupt zu bestätigen  
}

/*! \brief Endpunktbehandlung EP1 (IN)*/
void usb_ep1_in(void) //'Endpunkt 1 IN
{ 
  if (UEINTX & (1 << NAKINI)) // Sind Daten vom PC angefragt worden?
  {
    CBI(UEINTX, NAKINI); //NAKIN Interrupt Flag loeschen
    if (UEINTX & (1 << TXINI))   //Speicherbank frei?
    { 
      CBI(UEINTX, TXINI);  //loescht EP-Bank
#if defined FTDI  
      UEDATX = Ep1_buf[0]; // Sende immer die beiden Bytes zum 
      UEDATX = Ep1_buf[1]; // Modem und Line Status
#endif      
      if  (Ep1_flag == 1)
      {
#if defined FTDI
        Ep1_cnt += 2;
        for (int i = 2; i < Ep1_cnt; i++) UEDATX = Ep1_buf[i];
#else
        for (int i = 0; i < Ep1_cnt; i++) UEDATX = Ep1_buf[i];
#endif                
      }
      Ep1_flag = 0;
      Ep1_cnt = 0;
        
      CBI(UEINTX, FIFOCON); //FIFO wieder freigeben
      // zur Kontrolle: toggelt LED an PF4 (Teensy)
      if (PINF & (1<<PF4)) PORTF &= ~(1<<PF4); else PORTF |= (1<<PF4);
    }
  }
}

/*! \brief Endpunktbehandlung EP2 (OUT) */
void usb_ep2_out(void) //'Endpunkt 2 OUT
{
  if (UEINTX & (1 << RXOUTI)) //Daten vom PC angekommen?
  {
    CBI(UEINTX, RXOUTI); // Datenpaket bestaetigen (ACK)
    for (int i = 0; i < 64; i++)
    {
      if (UEINTX & (1 << RWAL)) //Daten im FIFO?
      {
	Ep2_buf[i] = UEDATX;   // Lese USB FIFO
	Ep2_cnt=i+1;
      }
      else break;
    }
    Ep2_flag = 1;
    CBI(UEINTX, FIFOCON); //FIFO wieder freigeben
  }
}
