/*! \file usb_small_lib_vendor_firmware_v1_2_c.c
********************************************************************************
 main source for using the small usb library using vendor class with bulk transfers
 
 www.weigu.lu/b/usb

 (inspired by the lib from S. Salewski (http://http://www.ssalewski.de)
 
 \author weigu www.weigu.lu

        In der Makefile muss der Chip und die Quarzfrequenz für das entsprechende
        Board festgelegt werden!

        Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):

        Eingaenge:   ADC0 (PF0) analog
        Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS

        AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):

        Eingaenge:   Temperatursensor an ADC0 (PF0) analog
        Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)

        Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):

        Eingaenge:   PC4
        Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS

        Informationen zur Funktionsweise:

        Hauptprogramm zur Nutzung der Vendor-Bibliothek
        Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
        Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
        abgewickelt. Alle USB-Routinen und Unterprogramme (wie zum Bsp. "USBINI")
        befinden sich in der Bibliothek "usb_srs_vendor_v1_2.c"

******************************************************************************/

/* Copyright (c) 2013   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lib/usb_srs_vendor_v1_2.h"

//Hier kann der FTDI Clone freigeschaltet werden
//#define FTDI            //falls nicht erwünscht, auskommentieren


/*! \brief  Timer1 Compare Match A Interrupt
    Toggelt LED an PD6 */
ISR (TIMER1_COMPA_vect)
{
  SBI (PIND,6); //Toggelt bei neuen Chips das Pin!  
}

/*! \brief Hauptprogramm
 */
int main(void)
{
  // Wirkung des Fusebit CKDIV ueberschreiben
  CLKPR = (1<<CLKPCE);  
  CLKPR = 0;
  // Timer1 initialisieren, Interrupt ein mal pro Sekunde
  // Vorteiler 1024, Zaehlschritte 7811-1 ergeben bei 16MHz 1 Sekunde
  TCCR1A = 0;
  TCCR1B = 0x0D;           // WGM12 = 1 Teiler = 1024
#if F_CPU == 8000000
  OCR1AH = high(3905);    // Achtung High zuerst!
  OCR1AL = low(3905);
#else
  OCR1AH = high(7811);    // Achtung High zuerst!
  OCR1AL = low(7811);
#endif
  TIMSK1 |= (1 << OCIE1A); // ;OCI1A Interrupt erlauben
  // AD-Wandler initialisieren
#if defined(__AVR_AT90USB162__) // Der AT90USB162 besitzt keinen A/D-Wandler
  DDRC &= ~(1<<DDC4)            // Es wird deshalb PC4 als Eingang eingelesen
  PORTC |= (1<<PC4);            // Pull-Up
#else                           // ADC initialisieren
  ADMUX = 0b01000000;   //REFS = 01 AVCC, ADLAR = 0 rechtsbuendig, ADC0
  ADCSRA = 0b10000111;  //ADEN = 1 (ADC einschalten), ADPS = 111
#endif                  //(16MHz/128 = 125kHz), andere Bits per default = 0
  // Ausgänge festlegen
#if defined(__AVR_AT90USB1287__) // Der AT90USBKEY hat 4 LEDs on board
  DDRD |= ((1<<DDD4) | (1<<DDD5) | (1<<DDD6) | (1<<DDD7)); // PD4-7 = Ausgang
#else                         
  DDRD |= (1<<DDD6);    // PD6 = Ausgang Teensy on board
  DDRB |= ((1<<DDB3) | (1<<DDB2) | (1<<DDB1) | (1<<DDB0)); // PB0-3 = Ausgang
#endif
  // USB starten (erlaubt auch Interrupts (sei))
  usb_init_device();	// [usb_srs.c]
  blinkflag = 1;
 
  while (1)
  {
    Ep1_cnt = 2;        // Es werden 2 Werte übergeben
#if defined(__AVR_AT90USB162__) // Der AT90USB162 besitzt keinen A/D-Wandler
    cli();
    if ((PINC & (1<<PC0)) == 0)
    {
      Ep1_buf[0] = 0;
      Ep1_buf[1] = 0;
    }  
    else
    {        
      Ep1_buf[0] = low(1023);
      Ep1_buf[1] = high(1023);
    }
    sei();
#else
    ADCSRA |= (1<<ADSC);   //ADC Wandlung starten (ADSC = 1)
    while ((ADCSRA &(1<<ADSC))); //und warten bis Wandlung fertig (ADSC = 0)
    cli();
    Ep1_buf[0] = ADCL;
    Ep1_buf[1] = ADCH;
    sei();
#endif
    Ep1_flag = 1;               //EP1 Flag setzen weil Daten vorhanden
    
    if (Ep2_flag == 1)
    {
      blinkflag = (Ep2_buf[0] & 0x40); //blinkflag ausmaskieren
#if defined(__AVR_AT90USB1287__) // Der AT90USBKEY hat 4 LEDs on board
      PORTD |= swap((Ep2_buf[0] & 0x0F)); //betroffene LEDs mit Maskierung setzen
      PORTD &= swap((Ep2_buf[0] | 0xF0)); //betroffene LEDs mit Maskierung loeschen
#else
      PORTB |= (Ep2_buf[0] & 0x0F); //betroffene LEDs mit Maskierung setzen
      PORTB &= (Ep2_buf[0] | 0xF0); //betroffene LEDs mit Maskierung loeschen
#endif
    Ep2_flag = 0;
    }
    if (blinkflag) TIMSK1 |= (1 << OCIE1A); // ;OCI1A Interrupt erlauben
    else TIMSK1 &= ~(1 << OCIE1A); // ;OCI1A Interrupt sperren
  }
}